@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.44 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmd.c,v 1.14 2009/08/07 03:38:03 william Exp $
  */
 
 #define _INCLUDE_FROM_CMD_C_
@@ -30,6 +30,7 @@
 #include "drawing.e"
 #include "dup.e"
 #include "file.e"
+#include "grid.e"
 #include "imgproc.e"
 #include "mainmenu.e"
 #include "mark.e"
@@ -85,7 +86,89 @@ void InsertCmd(PrevCmd, NextCmd, CmdPtr, ppFirstCmd, ppLastCmd)
    }
 }
 
-#ifdef _TGIF_WB2
+static
+char *GetCmdType(CmdType)
+   int CmdType;
+{
+   switch (CmdType) {
+   case CMD_COMPOSITE: return "CMD_COMPOSITE";
+   case CMD_NEW: return "CMD_NEW";
+   case CMD_DELETE: return "CMD_DELETE";
+   case CMD_MOVE: return "CMD_MOVE";
+   case CMD_STRETCH: return "CMD_STRETCH";
+   case CMD_ONE_TO_MANY: return "CMD_ONE_TO_MANY";
+   case CMD_MANY_TO_ONE: return "CMD_MANY_TO_ONE";
+   case CMD_REPLACE: return "CMD_REPLACE";
+   case CMD_GOTO_PAGE: return "CMD_GOTO_PAGE";
+   case CMD_WB_CLEARALL: return "CMD_WB_CLEARALL";
+   case CMD_CHAT_A_LINE: return "CMD_CHAT_A_LINE";
+   case CMD_WB_SLIDESHOW: return "CMD_WB_SLIDESHOW";
+   default: return "(unknown)";
+   }
+}
+
+static
+struct CmdRec *CreateCmdFromSerializationBuf(cmd_type, buf)
+   int cmd_type;
+   char *buf;
+{
+   struct CmdRec *cmd_ptr=NULL;
+
+   cmd_ptr = (struct CmdRec *)malloc(sizeof(struct CmdRec));
+   if (cmd_ptr == NULL) FailAllocMessage(); 
+   memset(cmd_ptr, 0, sizeof(struct CmdRec)); 
+
+   cmd_ptr->type = cmd_type;
+   cmd_ptr->serialization_buf = UtilStrDup(buf);
+   if (cmd_ptr->serialization_buf == NULL) FailAllocMessage();
+
+   return cmd_ptr;
+}
+
+void CopyAndInsertCmd(from_remote, serialization_buf, PrevCmd, NextCmd, CmdPtr,
+      ppFirstCmd, ppLastCmd)
+   int from_remote;
+   char *serialization_buf;
+   struct CmdRec *PrevCmd, *NextCmd, *CmdPtr;
+   struct CmdRec **ppFirstCmd, **ppLastCmd;
+   /* add CmdPtr between PrevCmd and NextCmd */
+   /* update *ppFirstCmd and *ppLastCmd */
+{
+   if ((!from_remote && CmdPtr->undone) || (from_remote && !CmdPtr->undone)) {
+      sprintf(gszMsgBox, "(%s) Found an %s command, type = %s.",
+            (from_remote ? "from_remote" : "local"),
+            (CmdPtr->undone ? "undone" : "not undone"),
+            GetCmdType(CmdPtr->type));
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+   } else {
+      struct CmdRec *cmd_ptr=NULL;
+
+#ifdef _TGIF_DBG_WB2
+      fprintf(stdout,
+            "<*><*><*>(%s) CopyAndInsertCmd() a command of type: %s.\n",
+            (from_remote ? "from_remote" : "local"),
+            GetCmdType(CmdPtr->type));
+      fflush(stdout);
+#endif /* _TGIF_DBG_WB2 */
+
+      cmd_ptr = CreateCmdFromSerializationBuf(CmdPtr->type, serialization_buf);
+
+      cmd_ptr->prev = PrevCmd;
+      cmd_ptr->next = NextCmd;
+
+      if (PrevCmd == NULL) {
+         (*ppFirstCmd) = cmd_ptr;
+      } else {
+         PrevCmd->next = cmd_ptr;
+      }
+      if (NextCmd == NULL) {
+         (*ppLastCmd) = cmd_ptr;
+      } else {
+         NextCmd->prev = cmd_ptr;
+      }
+   }
+}
+
 static
 void UnlinkCmd(CmdPtr, ppFirstCmd, ppLastCmd)
    struct CmdRec *CmdPtr;
@@ -102,7 +185,6 @@ void UnlinkCmd(CmdPtr, ppFirstCmd, ppLastCmd)
       CmdPtr->next->prev = CmdPtr->prev;
    }
 }
-#endif /* _TGIF_WB2 */
 
 void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
    struct CmdRec *CmdPtr;
@@ -158,6 +240,9 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
          next_sel = sel_ptr->next;
          free(sel_ptr);
       }
+      if (CmdPtr->subcmd != NULL) {
+         free(CmdPtr->subcmd);
+      }
       break;
    case CMD_STRETCH:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
@@ -230,19 +315,26 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
       }
       break;
    case CMD_CHAT_A_LINE:
-#ifdef _TGIF_WB2
-      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
-         next_sel = sel_ptr->next;
-         FreeObj(sel_ptr->obj);
-         free(sel_ptr);
+      if (CmdPtr->subcmd != NULL) {
+         UtilFree(CmdPtr->subcmd->detail.chat.buf);
+         free(CmdPtr->subcmd);
       }
-#endif /* _TGIF_WB2 */
+      break;
+   case CMD_WB_SLIDESHOW:
+      if (CmdPtr->subcmd != NULL) free(CmdPtr->subcmd);
+      break;
+   case CMD_WB_CLEARALL:
+      if (CmdPtr->subcmd != NULL) free(CmdPtr->subcmd);
       break;
    }
-#ifdef _TGIF_WB2
-   if (CmdPtr->sender_process_id != NULL) UtilFree(CmdPtr->sender_process_id);
-#endif /* _TGIF_WB2 */
-
+   if (gstWBInfo.do_whiteboard) {
+      if (CmdPtr->sender_process_id != NULL) {
+         UtilFree(CmdPtr->sender_process_id);
+      }
+      if (CmdPtr->serialization_buf != NULL) {
+         UtilFree(CmdPtr->serialization_buf);
+      }
+   }
    free(CmdPtr);
 }
 
@@ -396,12 +488,12 @@ int ObjHasFullID(obj_ptr, psz_full_id)
    char *psz_full_id;
 {
    if (*psz_full_id != '#') {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
-            "Unexpected *psz_full_id != '#' in ObjHasFullID()",
-            psz_full_id, gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
+               "Unexpected *psz_full_id != '#' in ObjHasFullID()",
+               psz_full_id, gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
    }
    psz_full_id = strchr(psz_full_id, ':');
    psz_full_id++;
@@ -422,24 +514,24 @@ int GetPositionFromFullID(psz_full_id)
    char *psz=NULL;
 
    if (*psz_full_id != '#') {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
-            "Unexpected *psz_full_id != '#' in GetPositionFromFullID()",
-            psz_full_id, gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s, psz_full_id = %s, pid = %ld.",
+               "Unexpected *psz_full_id != '#' in GetPositionFromFullID()",
+               psz_full_id, gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
    }
    psz = strchr(++psz_full_id, ':');
    *psz = '\0';
    if (sscanf(psz_full_id, "%d", &pos_to_match) != 1 ||
          pos_to_match == INVALID) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
-            "Format error in parsing pos_to_match in",
-            "GetPositionFromFullID()", "cannot get pos_to_match",
-            psz_full_id, gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
+               "Format error in parsing pos_to_match in",
+               "GetPositionFromFullID()", "cannot get pos_to_match",
+               psz_full_id, gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
    }
    *psz = ':';
 
@@ -554,7 +646,7 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
    for (index=0; index < CmdPtr->count_before; index++) {
       char *psz_full_id=(((char**)CmdPtr->pos_before)[index]), *psz=NULL;
       char pid_and_host[MAXSTRING];
-      int id=(-1), found=FALSE, pos_to_match=INVALID;
+      int id=(-1), found=FALSE, pos_to_match=INVALID, obj_is_tgifobj=FALSE;
 
       if (*psz_full_id == '#') {
          psz = strchr(++psz_full_id, ':');
@@ -562,13 +654,13 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
          if (!modify_before_pointers) {
             if (sscanf(psz_full_id, "%d", &pos_to_match) != 1 ||
                   pos_to_match == INVALID) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-               sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
-                     "Format error in parsing pos_to_match in",
-                     "SetupBeforePointers()", "cannot get pos_to_match",
-                     psz_full_id, gstWBInfo.pid);
-               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+               if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+                  sprintf(gszMsgBox, "%s %s (%s), psz_full_id = %s, pid = %ld.",
+                        "Format error in parsing pos_to_match in",
+                        "SetupBeforePointers()", "cannot get pos_to_match",
+                        psz_full_id, gstWBInfo.pid);
+                  MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+               }
             }
          }
          *psz++ = ':';
@@ -577,37 +669,44 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
       psz = strchr(psz_full_id, '/');
       *pid_and_host = '\0';
       if (psz == NULL) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-         sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
-               "Format error in parsing id in SetupBeforePointers()",
-               "cannot find ':'", psz_full_id, gstWBInfo.pid);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+         if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+            sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
+                  "Format error in parsing id in SetupBeforePointers()",
+                  "cannot find ':'", psz_full_id, gstWBInfo.pid);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         }
          ok = FALSE;
          break;
       } else {
          *psz = '\0';
          if (sscanf(psz_full_id, "%d", &id) == 1) {
             strcpy(pid_and_host, &psz[1]);
+            if (id == INVALID) {
+               obj_is_tgifobj = TRUE;
+            }
          } else {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-            sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
-                  "Format error in parsing id in SetupBeforePointers()",
-                  "cannot find id", psz_full_id, gstWBInfo.pid);
-            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+            if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+               sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
+                     "Format error in parsing id in SetupBeforePointers()",
+                     "cannot find id", psz_full_id, gstWBInfo.pid);
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            }
             ok = FALSE;
             break;
          }
          *psz = '/';
       }
       if (ok) {
-         for ( ; ok && obj_ptr != NULL; obj_ptr=obj_ptr->next, cur_pos++) {
+         for (obj_ptr=topObj, cur_pos=0 ; ok && obj_ptr != NULL;
+               obj_ptr=obj_ptr->next, cur_pos++) {
             if (pos_to_match == INVALID) {
                if (obj_ptr->creator_full_id == NULL) {
                   if (obj_ptr->id == id &&
                         strcmp(pid_and_host, gszLocalPID) == 0) {
                      found = TRUE;
+                  } else if (obj_ptr->id == INVALID &&
+                        CmdPtr->include_tgif_obj && obj_is_tgifobj) {
+                     found = TRUE;
                   }
                } else {
                   if (strcmp(obj_ptr->creator_full_id, psz_full_id) == 0) {
@@ -631,25 +730,25 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
          }
       }
       if (ok && !found) {
-#ifdef _TGIF_WB2
+         if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
 #ifdef _TGIF_DBG_WB2 /* debug, do not translate */
-         fprintf(stderr, "%s: '%s' %s, pid = %ld.\n\t(%s=%1d, %s='%s')\n",
-               "@*@ Cannot find object id", psz_full_id,
-               "in SetupBeforePointers()", gstWBInfo.pid, "id", id,
-               "pid_and_host", pid_and_host);
+            fprintf(stderr, "%s: '%s' %s, pid = %ld.\n\t(%s=%1d, %s='%s')\n",
+                  "@*@ Cannot find object id", psz_full_id,
+                  "in SetupBeforePointers()", gstWBInfo.pid, "id", id,
+                  "pid_and_host", pid_and_host);
 #endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
+         }
          ok = FALSE;
          break;
       }
    }
    if (ok && index != CmdPtr->count_before) {
-#ifdef _TGIF_WB2 /* debug, do not translate */
-      sprintf(gszMsgBox, "%s (%1d) %s (%1d) %s, pid = %ld.",
-            "Index", index, "!= CmdPtr->count_before", CmdPtr->count_before,
-            "in SetupBeforePointers()", gstWBInfo.pid);
-      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-#endif /* _TGIF_WB2 */
+      if (gstWBInfo.do_whiteboard) { /* debug, do not translate */
+         sprintf(gszMsgBox, "%s (%1d) %s (%1d) %s, pid = %ld.",
+               "Index", index, "!= CmdPtr->count_before", CmdPtr->count_before,
+               "in SetupBeforePointers()", gstWBInfo.pid);
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      }
       ok = FALSE;
    }
    if (!ok) {
@@ -677,9 +776,18 @@ int RedoDeleteCmd(CmdPtr)
    struct SelRec *sel_ptr;
 
    if (CmdPtr->serialized) {
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
       if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+         if (CmdPtr->include_tgif_obj) {
+            UnlinkObj(topObj);
+         }
          return FALSE;
       }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
    topSel = CmdPtr->top_before;
    botSel = CmdPtr->bot_before;
@@ -740,12 +848,23 @@ int UndoOrRedoMoveCmd(CmdPtr)
    int dx, dy;
 
    if (CmdPtr->serialized) {
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
       if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+         if (CmdPtr->include_tgif_obj) {
+            UnlinkObj(topObj);
+         }
          return FALSE;
       }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
-   dx = (CmdPtr->undone) ? CmdPtr->dx : -(CmdPtr->dx);
-   dy = (CmdPtr->undone) ? CmdPtr->dy : -(CmdPtr->dy);
+   dx = (CmdPtr->undone) ? CmdPtr->subcmd->detail.move.dx :
+           -(CmdPtr->subcmd->detail.move.dx);
+   dy = (CmdPtr->undone) ? CmdPtr->subcmd->detail.move.dy :
+           -(CmdPtr->subcmd->detail.move.dy);
 
    CopySel(CmdPtr->top_before, CmdPtr->count_before, &topSel, &botSel);
 
@@ -763,7 +882,7 @@ int UndoOrRedoMoveCmd(CmdPtr)
          for (obj_ptr=topObj; obj_ptr != NULL; obj_ptr=next_obj) {
             next_obj = obj_ptr->next;
             if (ObjHasFullID(obj_ptr,
-                  (char*)(int)(CmdPtr->pos_before[count]))) {
+                  (char*)(long)(CmdPtr->pos_before[count]))) {
                count++;
                sel_ptr->obj = obj_ptr;
                if (!obj_ptr->locked) MoveObj(obj_ptr, dx, dy);
@@ -837,9 +956,8 @@ void UpdateXPmObjectsInACmd(CmdPtr)
 }
 
 static
-int UpdatePixelInACmd(CmdPtr, old_color_str)
+int UpdatePixelInACmd(CmdPtr)
    struct CmdRec *CmdPtr;
-   char **old_color_str;
 {
    struct CmdRec *cmd_ptr;
    struct SelRec *sel_ptr;
@@ -848,12 +966,16 @@ int UpdatePixelInACmd(CmdPtr, old_color_str)
    switch (CmdPtr->type) {
    case CMD_COMPOSITE:
       for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
-         if (UpdatePixelInACmd(cmd_ptr, old_color_str)) changed = TRUE;
+         if (UpdatePixelInACmd(cmd_ptr)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_NEW:
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_DELETE: break;
@@ -861,16 +983,22 @@ int UpdatePixelInACmd(CmdPtr, old_color_str)
    case CMD_STRETCH: break;
    case CMD_REPLACE:
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_ONE_TO_MANY: break;
    case CMD_MANY_TO_ONE:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
-         if (UpdatePixel(sel_ptr->obj, old_color_str)) changed = TRUE;
+         if (UpdatePixel(sel_ptr->obj)) {
+            changed = TRUE;
+         }
       }
       break;
    case CMD_GOTO_PAGE: break;
@@ -886,8 +1014,7 @@ int RefreshColormap(nRedoing, CmdPtr)
     * This function mimics FlushColormap() in color.c.
     */
 {
-   int i=0, old_allocated_colors, changed=FALSE, saved_color_layers;
-   char **old_color_str=NULL;
+   int changed=FALSE, saved_color_layers;
    struct ObjRec *obj_ptr=NULL;
    struct PageRec *page_ptr=NULL;
    Colormap colormap=XCopyColormapAndFree(mainDisplay, mainColormap);
@@ -915,15 +1042,6 @@ int RefreshColormap(nRedoing, CmdPtr)
          }
       }
    }
-   old_color_str = (char**)malloc(maxColors*sizeof(char*));
-   if (old_color_str == NULL) FailAllocMessage();
-   old_allocated_colors = maxColors;
-   for (i = 0; i < maxColors; i++) {
-      old_color_str[i] =
-            (char*)malloc((strlen(colorMenuItems[i])+1)*sizeof(char));
-      if (old_color_str[i] == NULL) FailAllocMessage();
-      strcpy(old_color_str[i], colorMenuItems[i]);
-   }
    initColorDontReload = TRUE;
    CleanUpColors();
    XFreeColormap(mainDisplay, mainColormap);
@@ -945,28 +1063,26 @@ int RefreshColormap(nRedoing, CmdPtr)
 
       for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
          for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
-            if (UpdatePixel(obj_ptr, old_color_str)) {
+            if (UpdatePixel(obj_ptr)) {
                changed = TRUE;
             }
          }
       }
       for (cmd_ptr=CmdPtr->next; cmd_ptr != NULL; cmd_ptr=cmd_ptr->next) {
-         if (UpdatePixelInACmd(cmd_ptr, old_color_str)) {
+         if (UpdatePixelInACmd(cmd_ptr)) {
             changed = TRUE;
          }
       }
    } else {
       for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
          for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
-            if (UpdatePixel(obj_ptr, old_color_str)) {
+            if (UpdatePixel(obj_ptr)) {
                changed = TRUE;
             }
          }
       }
    }
    RestoreStatusStrings();
-   for (i = 0; i < old_allocated_colors; i++) free(old_color_str[i]);
-   free(old_color_str);
 
    DestroyPinnedMenu(MENU_COLOR);
    if (colorLayers) {
@@ -988,14 +1104,12 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
    int need_clear_and_redraw=FALSE;
    int *actual_pos_before=NULL;
 
-#ifdef _TGIF_WB2
 #ifdef _TGIF_DBG_WB2 /* debug, do not translate */
    if (gstWBInfo.do_whiteboard) {
       fprintf(stderr, "@@@ Beginning of UndoOrRedoReplaceCmd()...\n");
       fprintf(stderr, "\tCmdPtr->serialized=%1d...\n", CmdPtr->serialized);
    }
 #endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
    if (CmdPtr->serialized) {
       int shuffle=FALSE;
 
@@ -1003,15 +1117,19 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
          CmdPtr->first_redo_after_deserialize = FALSE;
          shuffle = TRUE;
       }
-#ifdef _TGIF_WB2
 #ifdef _TGIF_DBG_WB2 /* debug, do not translate */
       if (gstWBInfo.do_whiteboard) {
          fprintf(stderr, "In UndoOrRedoReplaceCmd(), shuffle=%1d.\n", shuffle);
       }
 #endif /* _TGIF_DBG_WB2 */
-#endif /* _TGIF_WB2 */
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
       if (shuffle) {
          if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+            if (CmdPtr->include_tgif_obj) {
+               UnlinkObj(topObj);
+            }
             return FALSE;
          }
          topSel = CmdPtr->top_before;
@@ -1036,6 +1154,9 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
             saved_bot_sel = CmdPtr->bot_before;
             CmdPtr->top_before = CmdPtr->bot_before = NULL;
             if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+               if (CmdPtr->include_tgif_obj) {
+                  UnlinkObj(topObj);
+               }
                return FALSE;
             }
             CmdPtr->top_after = CmdPtr->top_before;
@@ -1044,10 +1165,16 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
             CmdPtr->bot_before = saved_bot_sel;
          } else {
             if (!SetupBeforePointers(CmdPtr, FALSE, &actual_pos_before)) {
+               if (CmdPtr->include_tgif_obj) {
+                  UnlinkObj(topObj);
+               }
                return FALSE;
             }
          }
       }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
    LinkJustTheObjects(CmdPtr->top_before, CmdPtr->bot_before);
 
@@ -1183,11 +1310,20 @@ int UndoOrRedoOneToManyCmd(CmdPtr)
    int *actual_pos_before=NULL;
 
    if (CmdPtr->serialized) {
-     if (CmdPtr->undone) {
-        if (!SetupBeforePointers(CmdPtr, TRUE, &actual_pos_before)) {
-           return FALSE;
-        }
-     }
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
+      if (CmdPtr->undone) {
+         if (!SetupBeforePointers(CmdPtr, TRUE, &actual_pos_before)) {
+            if (CmdPtr->include_tgif_obj) {
+               UnlinkObj(topObj);
+            }
+            return FALSE;
+         }
+      }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
    if (CmdPtr->include_tgif_obj) {
       AddObj(NULL, topObj, tgifObj);
@@ -1203,7 +1339,7 @@ int UndoOrRedoOneToManyCmd(CmdPtr)
       pos_table = (CmdPtr->undone) ? CmdPtr->pos_before : CmdPtr->pos_after;
       max_count = (CmdPtr->undone) ? CmdPtr->count_before : CmdPtr->count_after;
 
-      if (CmdPtr->serialized && !CmdPtr->undone) {
+      if (CmdPtr->serialized && CmdPtr->undone) {
          pos_table = actual_pos_before;
       }
       pos = count = 0;
@@ -1247,7 +1383,7 @@ int UndoOrRedoOneToManyCmd(CmdPtr)
 
    pos = count = 0;
    sel_ptr = topSel;
-   if (CmdPtr->serialized && !CmdPtr->undone) {
+   if (CmdPtr->serialized && CmdPtr->undone) {
       pos_table = actual_pos_before;
    }
    for (obj_ptr=topObj; obj_ptr!=NULL; obj_ptr=next_obj, pos++) {
@@ -1315,6 +1451,22 @@ void UndoOrRedoGotoPageCmd(CmdPtr)
    ShowPage();
 }
 
+static
+void UndoOrRedoWBSlideShow(CmdPtr)
+   struct CmdRec *CmdPtr;
+{
+   int into_slideshow=CmdPtr->subcmd->detail.slideshow.into_slideshow;
+
+   if (CmdPtr->undone) {
+      /* executing the command */
+      if (!inSlideShow && into_slideshow) {
+         EnterSlideShow();
+      } else if (inSlideShow && !into_slideshow) {
+         LeaveSlideShow();
+      }
+   }
+}
+
 typedef struct tagUndoRedoRestoreDrawingModeRec {
    int cur_choice;
    int text_cursor_shown;
@@ -1383,9 +1535,10 @@ void UndoRedoRestoreDrawingMode(p_urrdmi)
    default: SetCurChoice(p_urrdmi->cur_choice); break;
    }
 }
-void UndoACmd(CmdPtr, HighLight)
+
+void UndoACmd(CmdPtr, HighLight, nPerformAction)
    struct CmdRec *CmdPtr;
-   int HighLight;
+   int HighLight, nPerformAction;
 {
    struct CmdRec *cmd_ptr;
 
@@ -1395,11 +1548,11 @@ void UndoACmd(CmdPtr, HighLight)
    case CMD_COMPOSITE:
       if (CmdPtr->first->type==CMD_MOVE || CmdPtr->first->type==CMD_STRETCH) {
          for (cmd_ptr=CmdPtr->last; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->prev) {
-            UndoACmd(cmd_ptr, FALSE);
+            UndoACmd(cmd_ptr, FALSE, nPerformAction);
          }
       } else {
          for (cmd_ptr=CmdPtr->last; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->prev) {
-            UndoACmd(cmd_ptr, TRUE);
+            UndoACmd(cmd_ptr, TRUE, nPerformAction);
          }
       }
       break;
@@ -1436,16 +1589,16 @@ void UndoCmd()
       return;
    }
    undoingOrRedoing = TRUE;
-   UndoACmd(curCmd, TRUE);
+   UndoACmd(curCmd, TRUE, TRUE);
    curCmd = curCmd->prev;
    undoingOrRedoing = FALSE;
 
    UndoRedoRestoreDrawingMode(&urrdmi);
 }
 
-int RedoACmd(CmdPtr, HighLight)
+int RedoACmd(CmdPtr, HighLight, nPerformAction)
    struct CmdRec *CmdPtr;
-   int HighLight;
+   int HighLight, nPerformAction;
 {
    int ok=TRUE;
    struct CmdRec *cmd_ptr;
@@ -1457,19 +1610,19 @@ int RedoACmd(CmdPtr, HighLight)
       if (CmdPtr->first->type==CMD_MOVE || CmdPtr->first->type==CMD_STRETCH) {
          for (cmd_ptr=CmdPtr->first->next; cmd_ptr!=NULL;
                cmd_ptr=cmd_ptr->next) {
-            if (!RedoACmd(cmd_ptr, FALSE)) {
+            if (!RedoACmd(cmd_ptr, FALSE, nPerformAction)) {
                ok = FALSE;
                break;
             }
          }
          if (ok) {
-            if (!RedoACmd(CmdPtr->first, TRUE)) {
+            if (!RedoACmd(CmdPtr->first, TRUE, nPerformAction)) {
                ok = FALSE;
             }
          }
       } else {
          for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
-            if (!RedoACmd(cmd_ptr, TRUE)) {
+            if (!RedoACmd(cmd_ptr, TRUE, nPerformAction)) {
                ok = FALSE;
                break;
             }
@@ -1484,6 +1637,7 @@ int RedoACmd(CmdPtr, HighLight)
    case CMD_ONE_TO_MANY: ok = UndoOrRedoOneToManyCmd(CmdPtr); break;
    case CMD_MANY_TO_ONE: ok = UndoOrRedoOneToManyCmd(CmdPtr); break;
    case CMD_GOTO_PAGE: UndoOrRedoGotoPageCmd(CmdPtr); break;
+   case CMD_WB_SLIDESHOW: UndoOrRedoWBSlideShow(CmdPtr); break;
    }
    CmdPtr->undone = FALSE;
    return ok;
@@ -1515,7 +1669,7 @@ void RedoCmd()
       curCmd = curCmd->next;
    }
    undoingOrRedoing = TRUE;
-   RedoACmd(curCmd, TRUE);
+   RedoACmd(curCmd, TRUE, TRUE);
    undoingOrRedoing = FALSE;
 
    UndoRedoRestoreDrawingMode(&urrdmi);
@@ -1776,31 +1930,70 @@ void FreeAfterSel(CmdPtr)
    CmdPtr->top_after = CmdPtr->bot_after = NULL;
 }
 
+static
+void PrepareExtendedSerializationInfoForRecordCmd(SubCmdPtr, TopSel, BotSel,
+      NumObjs, psi)
+   struct SubCmdRec *SubCmdPtr;
+   struct SelRec *TopSel, *BotSel;
+   int NumObjs;
+   SerializationInfo *psi;
+{
+   psi->subcmd = SubCmdPtr;
+   psi->top_sel = TopSel;
+   psi->bot_sel = BotSel;
+   psi->num_objs = NumObjs;
+
+   psi->include_tgif_obj = recordCmdIncludeTgifObj;
+   psi->new_colormap = recordCmdUsesNewColormap;
+   psi->logical_clock = gstWBInfo.logical_clock;
+   psi->sender_process_id = gszLocalPID;
+
+   psi->top_before = topSelBeforeInCmd;
+   psi->bot_before = botSelBeforeInCmd;
+   psi->pos_before = stackingPosition;
+   psi->count_before = stackingCount;
+   psi->pos_before_has_ids = stackingPositionHasIds;
+}
+
 void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    int CmdType, NumObjs;
-   struct SelRec *TopSel, *BotSel;
    struct SubCmdRec *SubCmdPtr;
+   struct SelRec *TopSel, *BotSel;
 {
    int logical_clock=0;
-#ifdef _TGIF_WB2
-   int wb_send_data_failed=FALSE;
-#endif /* _TGIF_WB2 */
+   int wb_send_data_failed=FALSE, plain_wb_data_sz=0;
+   char *psz_wb_data=NULL, *psz_plain_wb_data=NULL;
 
    if (gstWBInfo.do_whiteboard) {
-#ifdef _TGIF_WB2
-      char *psz_wb_data=NULL;
+      if (gstWBInfo.dont_serialize || CmdType == CMD_GOTO_PAGE) {
+         /*
+          * One of the important thing that happens in SerializeCmd()
+          *         is that the local logical clock got incremented.
+          *         If we don't increment the logical clock, we may
+          *         create a new command that have the same logical clock
+          *         as the one created last time we got here and that's
+          *         illegal. Therefore, we must increment the logical clock.
+          */
+         logical_clock = gstWBInfo.logical_clock++;
+      } else {
+         SerializationInfo si;
 
-      serializingFile = TRUE;
+         memset(&si, 0, sizeof(SerializationInfo));
+         PrepareExtendedSerializationInfoForRecordCmd(SubCmdPtr, TopSel, BotSel,
+               NumObjs, &si);
+         
+         serializingFile = TRUE;
 
-      if ((psz_wb_data=SerializeCmd(CmdType, SubCmdPtr, TopSel, BotSel,
-            NumObjs, &logical_clock)) != NULL) {
-         if (!SendWBData(psz_wb_data,logical_clock)) {
-            wb_send_data_failed = TRUE;
+         psz_plain_wb_data = NULL;
+         plain_wb_data_sz = 0;
+         if ((psz_wb_data=SerializeCmd(CmdType, &si, &logical_clock,
+               &psz_plain_wb_data, &plain_wb_data_sz)) != NULL) {
+            if (!SendWBData(psz_wb_data,logical_clock)) {
+               wb_send_data_failed = TRUE;
+            }
          }
-         SerializeFreeBuf(psz_wb_data);
+         serializingFile = FALSE;
       }
-      serializingFile = FALSE;
-#endif /* _TGIF_WB2 */
    } else {
       if (historyDepth == 0) return;
 
@@ -1852,6 +2045,28 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
       curCmd->sender_process_id = UtilStrDup(gszLocalPID);
       if (curCmd->sender_process_id == NULL) FailAllocMessage();
 
+      if (!wb_send_data_failed) {
+         /*
+          * FindShadowCmdInsertionPoint() examine the logical clock of the new
+          *       command and decides where to insert it in the shadow cmd list.
+          */
+         FindCmdInsertionPoint(curCmd, &immed_right_cmd);
+
+         if (immed_right_cmd == NULL) {
+            /* append */
+            CopyAndInsertCmd(FALSE,
+                  (psz_plain_wb_data==NULL ? psz_wb_data : psz_plain_wb_data),
+                  gstWBInfo.last_shadow_cmd, NULL, curCmd,
+                  &gstWBInfo.first_shadow_cmd, &gstWBInfo.last_shadow_cmd);
+         } else {
+            /* insert */
+            CopyAndInsertCmd(FALSE,
+                  (psz_plain_wb_data==NULL ? psz_wb_data : psz_plain_wb_data),
+                  immed_right_cmd->prev, immed_right_cmd, curCmd,
+                  &gstWBInfo.first_shadow_cmd, &gstWBInfo.last_shadow_cmd);
+         }
+      }
+      immed_right_cmd = NULL;
       /*
        * FindCmdInsertionPoint() examine the logical clock of the new command
        *       and decides where to insert it.
@@ -1877,24 +2092,50 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    case CMD_NEW: break;
    case CMD_DELETE: break;
    case CMD_MOVE:
-      curCmd->dx = SubCmdPtr->detail.mv->dx;
-      curCmd->dy = SubCmdPtr->detail.mv->dy;
+      curCmd->subcmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
+      if (curCmd->subcmd == NULL) FailAllocMessage();
+      memset(curCmd->subcmd, 0, sizeof(struct SubCmdRec));
+      curCmd->subcmd->detail.move.dx = SubCmdPtr->detail.move.dx;
+      curCmd->subcmd->detail.move.dy = SubCmdPtr->detail.move.dy;
       break;
    case CMD_STRETCH: FreeAfterSel(curCmd); break;
    case CMD_REPLACE: FreeAfterSel(curCmd); break;
    case CMD_ONE_TO_MANY: break;
    case CMD_MANY_TO_ONE: break;
    case CMD_GOTO_PAGE: curCmd->count_after = NumObjs; break;
-   case CMD_WB_CLEARALL: break;
+   case CMD_WB_CLEARALL:
+      curCmd->subcmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
+      if (curCmd->subcmd == NULL) FailAllocMessage();
+      memset(curCmd->subcmd, 0, sizeof(struct SubCmdRec));
+      curCmd->subcmd->detail.clearall.page_style =
+            SubCmdPtr->detail.clearall.page_style;
+      curCmd->subcmd->detail.clearall.print_mag =
+            SubCmdPtr->detail.clearall.print_mag;
+      break;
+   case CMD_CHAT_A_LINE:
+      /* intentionally forget what has been chatted */
+      break;
+   case CMD_WB_SLIDESHOW:
+      curCmd->subcmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
+      if (curCmd->subcmd == NULL) FailAllocMessage();
+      memset(curCmd->subcmd, 0, sizeof(struct SubCmdRec));
+      curCmd->subcmd->detail.slideshow.into_slideshow =
+            SubCmdPtr->detail.slideshow.into_slideshow;
+      break;
    }
-#ifdef _TGIF_WB2
-   if (wb_send_data_failed) {
-      UndoACmd(curCmd, FALSE);
-      UnlinkCmd(curCmd, &gstWBInfo.first_cmd, &gstWBInfo.last_cmd);
-      DeleteARedoRecord(curCmd, (-1.0), (-1.0));
+   if (gstWBInfo.do_whiteboard) {
+      if (wb_send_data_failed) {
+         UndoACmd(curCmd, FALSE, TRUE);
+         UnlinkCmd(curCmd, &gstWBInfo.first_cmd, &gstWBInfo.last_cmd);
+         DeleteARedoRecord(curCmd, (-1.0), (-1.0));
+      }
+      if (psz_wb_data != NULL) {
+         SerializeFreeBuf(psz_wb_data);
+      }
+      if (psz_plain_wb_data != NULL) {
+         UtilFree(psz_plain_wb_data);
+      }
    }
-#endif /* _TGIF_WB2 */
-
    curCmd = lastCmd;
 }
 
@@ -1989,8 +2230,39 @@ void ChangeReplaceOneCmdToDeleteCmd()
 void RecordWBClearAll()
 {
    if (gstWBInfo.do_whiteboard) {
+      struct SubCmdRec subcmd;
+
+      memset(&subcmd, 0, sizeof(struct SubCmdRec));
+      subcmd.detail.clearall.page_style = pageStyle;
+      subcmd.detail.clearall.print_mag = printMag;
+
       PrepareToRecord(CMD_WB_CLEARALL, NULL, NULL, 0);
-      RecordCmd(CMD_WB_CLEARALL, NULL, NULL, NULL, 0);
+      RecordCmd(CMD_WB_CLEARALL, &subcmd, NULL, NULL, 0);
+   }
+}
+
+void RecordWBSlideShow(into_slideshow)
+   int into_slideshow;
+{
+   if (gstWBInfo.do_whiteboard) {
+      if (pageLayoutMode == PAGE_STACK && firstPage != lastPage) {
+         struct SubCmdRec subcmd;
+
+         memset(&subcmd, 0, sizeof(struct SubCmdRec));
+         subcmd.detail.slideshow.into_slideshow = into_slideshow;
+
+         PrepareToRecord(CMD_WB_SLIDESHOW, NULL, NULL, 0);
+         RecordCmd(CMD_WB_SLIDESHOW, &subcmd, NULL, NULL, 0);
+      }
+   }
+}
+
+void RecordWBChatALine(pSubCmdPtr)
+   struct SubCmdRec *pSubCmdPtr;
+{
+   if (gstWBInfo.do_whiteboard) {
+      PrepareToRecord(CMD_CHAT_A_LINE, NULL, NULL, 0);
+      RecordCmd(CMD_CHAT_A_LINE, pSubCmdPtr, NULL, NULL, 0);
    }
 }
 
