@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.25 2009/06/15 19:34:23 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.28 2009/10/22 00:00:17 cvsps Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -440,6 +440,11 @@
 #define CMDID_EXTENDSEGMENT           498
 #define CMDID_TOGGLETIGHTERSPLINES    499
 
-#define MAXCMDIDS 500
+#define CMDID_INSERTSYMBOL             500
+#define CMDID_TOGGLERIGHTMARGIN        501
+#define CMDID_SPECIFYRIGHTMARGIN       502
+#define CMDID_CHANGEALLSELFONTOVERLINE 503
+
+#define MAXCMDIDS 504
 
 #endif /*_CMDIDS_H_*/
