@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.16 2009/08/13 01:14:20 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/color.c,v 1.17 2009/11/02 17:03:54 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_COLOR_C_
@@ -1368,7 +1368,7 @@ void ChangeAllSelColor(ColorIndex, HighLight)
    if (changed) {
       int need_to_update_sel_bbox=FALSE;
 
-      if (colorLayers) {
+      if (!PRTGIF && colorLayers) {
          struct SelRec *next_sel;
 
          for (sel_ptr=topSel; sel_ptr != NULL; sel_ptr=next_sel) {
@@ -1391,6 +1391,9 @@ void ChangeAllSelColor(ColorIndex, HighLight)
                free(sel_ptr);
             }
          }
+         if (needToRedrawColorWindow) {
+            RedrawColorWindow();
+         }
       }
       SetFileModified(TRUE);
       RedrawAnArea(botObj, selLtX-GRID_ABS_SIZE(1),
