@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.38 2003/07/22 23:28:16 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/eps.c,v 1.8 2009/05/28 03:53:11 william Exp $
  */
 
 #define _INCLUDE_FROM_EPS_C_
@@ -572,13 +572,16 @@ int ConvertTiffToXbm(fp, tiff_offset, tiff_sz, xbm_fname, xbm_sz)
    if (MkTempFile(xbm_fname, xbm_sz, tmpDir, TOOL_NAME) == NULL) {
       return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
    }
-   if ((xbm_fp=fopen(xbm_fname, "w")) == NULL) {
-      return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
-   }
    if (!tiffToXbmCmdInitialized) {
       strcpy(tiffToXbmCmd, "tifftopnm %s | pgmtopbm | pbmtoxbm");
    }
    sprintf(cmd, tiffToXbmCmd, tiff_fname);
+   if (!FindProgramInPath(cmd, NULL, FALSE)) {
+      return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
+   }
+   if ((xbm_fp=fopen(xbm_fname, "w")) == NULL) {
+      return ErrorInConvertTiffToXbm(fp, NULL, NULL, NULL);
+   }
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM), cmd);
    SetStringStatus(gszMsgBox);
    if (mainDisplay != NULL) XSync(mainDisplay, False);
@@ -767,7 +770,7 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
 {
    FILE *pFile=NULL, *pPipe=NULL;
    char *pszCmd=NULL, szBuf[MAXSTRING];
-   int bytes_read=0;
+   int bytes_read=0, watch_cursor=watchCursorOnMainWindow;
 
    if (MkTempFile(pszXbmPath, file_buf_sz, tmpDir, TOOL_NAME) == NULL) {
       return FALSE;
@@ -778,6 +781,11 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
       FailAllocMessage();
       return FALSE;
    }
+   sprintf(pszCmd, psToXbmCmd, pszPsPath);
+   if (!FindProgramInPath(pszCmd, NULL, FALSE)) {
+      free(pszCmd);
+      return FALSE;
+   }
    if ((pFile=fopen(pszXbmPath,"w")) == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_CANNOT_OPEN_FILE_FOR_WRITING),
             pszXbmPath);
@@ -785,7 +793,6 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
       free(pszCmd);
       return FALSE;
    }
-   sprintf(pszCmd, psToXbmCmd, pszPsPath);
    sprintf(gszMsgBox, TgLoadCachedString(CSTID_EXECUTING_GIVEN_PROGRAM),
          pszCmd);
    SetStringStatus(gszMsgBox);
@@ -798,6 +805,10 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
       unlink(pszXbmPath);
       return FALSE;
    }
+   if (!watch_cursor) {
+      SetWatchCursor(drawWindow);
+      SetWatchCursor(mainWindow);
+   }
    writeFileFailed = FALSE;
    while ((bytes_read=fread(szBuf, sizeof(char), sizeof(szBuf), pPipe)) > 0) {
       if ((int)fwrite(szBuf, sizeof(char), bytes_read, pFile) <= 0) {
@@ -806,6 +817,10 @@ int ConvertPsToXbm(pszPsPath, pszXbmPath, file_buf_sz)
       }
    }
    pclose(pPipe);
+   if (!watch_cursor) {
+      SetDefaultCursor(mainWindow);
+      ShowCursor();
+   }
    SetStringStatus(TgLoadCachedString(CSTID_DOTS_DONE));
    free(pszCmd);
    fclose(pFile);
@@ -1740,6 +1755,10 @@ struct ObjRec *CreateEPSObj(FileName, ImageW, ImageH, bitmap, image,
    obj_ptr->bbox.rby = obj_ptr->obbox.rby = h + drawOrigY;
    obj_ptr->type = OBJ_XBM;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
