@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/inmethod.c,v 4.10 2002/01/05 02:48:02 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/inmethod.c,v 1.7 2009/10/02 15:31:24 william Exp $
  */
 
 #define _INCLUDE_FROM_INMETHOD_C_
@@ -34,20 +34,46 @@
 #ifndef _NO_XIM
 #include "convxim.e"
 #endif /* ~_NO_XIM */
+#ifndef _NO_TGTWB5
+#include "tgtwb5.e"
+#endif /* ~_NO_TGTWB5 */
+
+#include "dialog.e"
 #include "inmethod.e"
+#include "msg.e"
 #include "util.e"
 #include "setup.e"
+#include "strtbl.e"
+#include "tidget.e"
+
+#ifndef _NO_DL_SUPPORT
+#define CLOSE_DL(handle) dlclose(handle)
+#define OPEN_DL(path,flag) dlopen((path),(flag))
+#define GET_DL_SYM(handle,symbol) dlsym((handle),(symbol))
+#ifndef RTLD_NOW
+#define OPEN_DL_MODE 1
+#else /* RTLD_NOW */
+#define OPEN_DL_MODE (RTLD_NOW|RTLD_GLOBAL)
+#endif /* ~RTLD_NOW */
+#else /* _NO_DL_SUPPORT */
+#define CLOSE_DL(handle)
+#define OPEN_DL(path,flag) NULL
+#define GET_DL_SYM(handle,symbol) NULL
+#define OPEN_DL_MODE 0
+#endif /* ~_NO_DL_SUPPORT */
 
 int gnInputMethod=TGIM_NONE; /* one of TGIM_* */
 int gnOverTheSpot=FALSE;
 
 int cmdLineHasInputMethod=FALSE;
+int cmdLineDoubleByteInputMethod=FALSE;
 char cmdLineInputMethod[MAXSTRING];
 
 static int gnInputMethodIndex=INVALID;
+static int gnSingleOrDoubleByteInputMethod=0;
 
 typedef void (CleanUpFunc)ARGS_DECL((Display *dpy, Window win));
-typedef int (InitFunc)ARGS_DECL((Display *dpy, Window win));
+typedef int (InitFunc)ARGS_DECL((Display *dpy, Window win, char *arg));
 typedef int (FocusInFunc)ARGS_DECL((Display *dpy, Window win));
 typedef int (FocusOutFunc)ARGS_DECL((Display *dpy, Window win));
 typedef int (SendKeyFunc)ARGS_DECL((Display *dpy, Window win,
@@ -75,9 +101,10 @@ typedef int (HandleLookupStringFunc)ARGS_DECL((Display *dpy, Window win,
                                                int buf_size, KeySym *p_key_sym,
                                                int *pn_has_ch));
 
-typedef struct IMInfoRec {
+typedef struct tagIMInfo {
    char *pszName;
    int nInputMethod;
+   int nSingleDouble;
    CleanUpFunc *pCleanUpFunc;
    InitFunc *pInitFunc;
    FocusInFunc *pFocusInFunc;
@@ -96,15 +123,18 @@ typedef struct IMInfoRec {
    HandleNewCurTextFunc *pHandleNewCurTextFunc;
    ExpectLookupStringFunc *pExpectLookupStringFunc;
    HandleLookupStringFunc *pHandleLookupStringFunc;
-} * IMInfoPtr;
+} IMInfo;
+
+static IMInfo *gpIMInfo=NULL;
 
 /* --------------------- xcin Routines --------------------- */
 
 #ifndef _NO_XCIN
 static
-int XcinInit(dpy, win)
+int XcinInit(dpy, win, arg)
    Display *dpy;
    Window win;
+   char *arg;
 {
    return TRUE;
 }
@@ -150,9 +180,10 @@ int XcinExpectCM(dpy, win)
 
 #ifndef _NO_CHINPUT
 static
-int ChinputInit(dpy, win)
+int ChinputInit(dpy, win, arg)
    Display *dpy;
    Window win;
+   char *arg;
 {
    return (HZclientInit(dpy)==0);
 }
@@ -233,9 +264,10 @@ void KinputCleanUp(dpy, win)
 }
 
 static
-int KinputInit(dpy, win)
+int KinputInit(dpy, win, arg)
    Display *dpy;
    Window win;
+   char *arg;
 {
    char *c_ptr=NULL;
 
@@ -308,7 +340,8 @@ int KinputExpectCM(dpy, win)
    Display *dpy;
    Window win;
 {
-   return (win==drawWindow);
+   return (win == drawWindow ||
+         (TidgetHasFocus() && win == TidgetGetFocusWindow()));
 }
 
 static
@@ -328,7 +361,8 @@ int KinputActiveOnCntrlSpace(dpy, win)
    Display *dpy;
    Window win;
 {
-   return (win==drawWindow);
+   return (win == drawWindow ||
+         (TidgetHasFocus() && win == TidgetGetFocusWindow()));
 }
 
 static
@@ -376,15 +410,17 @@ void XimCleanUp(dpy, win)
    Display *dpy;
    Window win;
 {
-   XIMClose();
+   XIMCleanUp();
 }
 
 static
-int XimInit(dpy, win)
+int XimInit(dpy, win, arg)
    Display *dpy;
    Window win;
+   char *arg;
 {
    char *c_ptr=NULL;
+   XIMInit();
    gnOverTheSpot = FALSE;
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "PreeditType")) != NULL &&
          UtilStrICmp(c_ptr, "overthespot") == 0) {
@@ -394,6 +430,24 @@ int XimInit(dpy, win)
    return TRUE;
 }
 
+static
+int XimFocusIn(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   XIMSetICFocus(dpy, win);
+   return TRUE;
+}
+
+static
+int XimFocusOut(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   XIMUnsetICFocus(win);
+   return TRUE;
+}
+
 static
 int XimDeactiveOnCreateText(dpy, win)
    Display *dpy;
@@ -407,7 +461,7 @@ int XimHandleCreateText(dpy, win)
    Display *dpy;
    Window win;
 {
-   XIMUnsetICFocus();
+   XIMUnsetICFocus(win);
    return TRUE;
 }
 
@@ -416,7 +470,8 @@ int XimExpectNextEvent(dpy, win)
    Display *dpy;
    Window win;
 {
-   return (win==drawWindow);
+   return (win == drawWindow ||
+         (TidgetHasFocus() && win == TidgetGetFocusWindow()));
 }
 
 static
@@ -443,7 +498,8 @@ int XimExpectLookupString(dpy, win)
    Display *dpy;
    Window win;
 {
-   return (win==drawWindow);
+   return (win == drawWindow ||
+         (TidgetHasFocus() && win == TidgetGetFocusWindow()));
 }
 
 static XIMStatus c_stat;
@@ -474,25 +530,98 @@ int XimTellCursorPos(dpy, win, cur_x, cur_y)
 }
 #endif /* ~_NO_XIM */
 
+/* --------------------- tgtwb5 Routines --------------------- */
+
+#ifndef _NO_TGTWB5
+static
+void Tgtwb5CleanUp(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   Tgtwb5_CleanUp(dpy, win);
+}
+
+static
+int Tgtwb5Init(dpy, win, arg)
+   Display *dpy;
+   Window win;
+   char *arg;
+{
+   return Tgtwb5_Init(dpy, win, arg);
+}
+
+static
+int Tgtwb5SendKey(dpy, win, key_ev, buf)
+   Display *dpy;
+   Window win;
+   XKeyEvent *key_ev;
+   char *buf;
+{
+   return Tgtwb5_SendKey(dpy, win, key_ev, buf);
+}
+
+static
+int Tgtwb5ExpectCM(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   return FALSE;
+}
+
+static
+int Tgtwb5ActiveOnCntrlSpace(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   return (win == drawWindow ||
+         (TidgetHasFocus() && win == TidgetGetFocusWindow()));
+}
+
+static
+int Tgtwb5HandleCntrlSpace(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   return Tgtwb5_HandleCntrlSpace(dpy, win);
+}
+
+static
+int Tgtwb5DeactiveOnCreateText(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   return TRUE;
+}
+
+static
+int Tgtwb5HandleCreateText(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   Tgtwb5_HandleCreateText(dpy, win);
+   return TRUE;
+}
+#endif /* ~_NO_XCIN */
+
 /* --------------------- Global Routines --------------------- */
 
-static struct IMInfoRec gstIMInfo[] = {
+static struct tagIMInfo gstIMInfo[] = {
 #ifndef _NO_XCIN
-   { "xcin", TGIM_XCIN,
+   { "xcin", TGIM_XCIN, TGIM_DBIM,
      NULL, XcinInit, XcinFocusIn, XcinFocusOut,
      XcinSendKey, NULL, XcinExpectCM, NULL,
      NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    },
 #endif /* ~_NO_XCIN */
 #ifndef _NO_CHINPUT
-   { "chinput", TGIM_CHINPUT,
+   { "chinput", TGIM_CHINPUT, TGIM_DBIM,
      NULL, ChinputInit, ChinputFocusIn, ChinputFocusOut,
      ChinputSendKey, NULL, ChinputExpectCM, ChinputHandleCM,
      NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
    },
 #endif /* ~_NO_CHINPUT */
 #ifndef _NO_KINPUT
-   { "kinput2", TGIM_KINPUT,
+   { "kinput2", TGIM_KINPUT, TGIM_DBIM,
      KinputCleanUp, KinputInit, KinputFocusIn, KinputFocusOut,
      KinputSendKey, KinputConvProperty, KinputExpectCM, KinputHandleCM,
      KinputActiveOnCntrlSpace, KinputHandleCntrlSpace,
@@ -501,62 +630,195 @@ static struct IMInfoRec gstIMInfo[] = {
    },
 #endif /* ~_NO_KINPUT */
 #ifndef _NO_XIM
-   { "xim", TGIM_XIM,
-     XimCleanUp, XimInit, NULL, NULL,
+   { "xim", TGIM_XIM, (TGIM_DBIM|TGIM_SBIM),
+     XimCleanUp, XimInit, XimFocusIn, XimFocusOut,
      NULL, NULL, NULL, NULL, NULL, NULL,
      XimDeactiveOnCreateText, XimHandleCreateText,
      XimTellCursorPos, XimExpectNextEvent, XimHandleNextEvent,
      XimHandleNewCurText, XimExpectLookupString, XimHandleLookupString
    },
 #endif /* ~_NO_XIM */
-   { NULL, TGIM_NONE, NULL, NULL, NULL, NULL,
+#ifndef _NO_TGTWB5
+   { "tgtwb5", TGIM_TGTWB5, TGIM_DBIM,
+     Tgtwb5CleanUp, Tgtwb5Init, NULL, NULL,
+     Tgtwb5SendKey, NULL, Tgtwb5ExpectCM, NULL,
+     Tgtwb5ActiveOnCntrlSpace, Tgtwb5HandleCntrlSpace,
+     Tgtwb5DeactiveOnCreateText, Tgtwb5HandleCreateText,
+     NULL, NULL, NULL, NULL, NULL, NULL
+   },
+#endif /* ~_NO_TGTWB5 */
+   { NULL, TGIM_NONE, 0, NULL, NULL, NULL, NULL,
      NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
      NULL, NULL, NULL, NULL, NULL
    }
 };
 
+static void *gpLibIdnHandle=NULL;
+static Tgim_stringprep_convert_Func *gpfn_stringprep_convert=NULL;
+
+static
+void CleanUpTgim_stringprep_convert()
+{
+   if (gpLibIdnHandle != NULL) {
+      gpfn_stringprep_convert = NULL;
+      CLOSE_DL(gpLibIdnHandle);
+      gpLibIdnHandle = NULL;
+   }
+}
+
+static
+void InitTgim_stringprep_convert()
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   gpLibIdnHandle = OPEN_DL("libidn.so", OPEN_DL_MODE);
+   if (gpLibIdnHandle != NULL) {
+      gpfn_stringprep_convert =
+            (Tgim_stringprep_convert_Func*)GET_DL_SYM(gpLibIdnHandle,
+            "stringprep_convert");
+      if (gpfn_stringprep_convert == NULL) {
+         CLOSE_DL(gpLibIdnHandle);
+         gpLibIdnHandle = NULL;
+      }
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+}
+
+int Tgim_has_stringprep_convert()
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   if (gpLibIdnHandle != NULL && gpfn_stringprep_convert != NULL) {
+      return TRUE;
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+   return FALSE;
+}
+
+char *Tgim_stringprep_convert(const char *str, const char *to_codeset,
+      const char *from_codeset)
+{
+#if (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ))
+   if (gpLibIdnHandle != NULL && gpfn_stringprep_convert != NULL) {
+       return (gpfn_stringprep_convert)(str, to_codeset, from_codeset);
+   }
+#endif /* (!defined(_NO_IDNLIB) || defined(HAVE_LIBZ)) */
+   return NULL;
+}
+
 void CleanUpInputMethods()
 {
+   CleanUpTgim_stringprep_convert();
    if (gnInputMethodIndex != INVALID &&
          gstIMInfo[gnInputMethodIndex].pCleanUpFunc != NULL) {
       (gstIMInfo[gnInputMethodIndex].pCleanUpFunc)(mainDisplay, mainWindow);
    }
    gnInputMethodIndex = INVALID;
    cmdLineHasInputMethod = FALSE;
+   cmdLineDoubleByteInputMethod = FALSE;
    *cmdLineInputMethod = '\0';
+   gpIMInfo = NULL;
+   gnSingleOrDoubleByteInputMethod = 0;
 }
 
 int InitInputMethods()
 {
-   char *psz=NULL;
+   char *psz=NULL, single_double=0;
 
    gnInputMethod = gnInputMethodIndex = INVALID;
    if (cmdLineHasInputMethod) {
       UtilTrimBlanks(cmdLineInputMethod);
       psz = cmdLineInputMethod;
+      single_double = (cmdLineDoubleByteInputMethod ? TGIM_DBIM : TGIM_SBIM);
    } else {
       psz = XGetDefault(mainDisplay, TOOL_NAME, "DoubleByteInputMethod");
+      if (psz == NULL) {
+         psz = XGetDefault(mainDisplay, TOOL_NAME, "SingleByteInputMethod");
+         single_double = TGIM_SBIM;
+      } else {
+         single_double = TGIM_DBIM;
+      }
    }
+   gnSingleOrDoubleByteInputMethod = single_double;
+
    if (psz != NULL) {
-      struct IMInfoRec *pIMInfo=gstIMInfo;
+      IMInfo *pIMInfo=gstIMInfo;
+      char *psz_comma=strchr(psz, ',');
       int i=0;
 
+      if (psz_comma != NULL) *psz_comma = '\0';
       for ( ; pIMInfo->pszName != NULL; pIMInfo++, i++) {
-         if (UtilStrICmp(psz, pIMInfo->pszName) == 0) {
+         if (UtilStrICmp(psz, pIMInfo->pszName) == 0 &&
+               (single_double & pIMInfo->nSingleDouble) == single_double) {
             gnInputMethod = pIMInfo->nInputMethod;
             gnInputMethodIndex = i;
-            if (gstIMInfo[gnInputMethodIndex].pInitFunc != NULL &&
-                  !((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
-                  mainWindow))) {
-               /* may not be a problem */
+            gpIMInfo = pIMInfo;
+            if (gstIMInfo[gnInputMethodIndex].pInitFunc != NULL) {
+               if (psz_comma == NULL) {
+                  if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
+                        mainWindow, NULL))) {
+                     /* may not be a problem */
+                  }
+               } else {
+                  if (!((gstIMInfo[gnInputMethodIndex].pInitFunc)(mainDisplay,
+                        mainWindow, &psz_comma[1]))) {
+                     /* may not be a problem */
+                  }
+               }
             }
             break;
          }
       }
+      if (psz_comma != NULL) *psz_comma++ = ',';
+
+      if (gnInputMethod == INVALID) {
+         if (single_double == TGIM_DBIM) {
+            fprintf(stderr, TgLoadString(STID_DBIM_UNSUPPORTED), psz);
+         } else {
+            fprintf(stderr, TgLoadString(STID_SBIM_UNSUPPORTED), psz);
+         }
+         fprintf(stderr, "\n");
+      }
+   }
+   if (gnSingleOrDoubleByteInputMethod == TGIM_SBIM &&
+         gnInputMethod != INVALID) {
+      InitTgim_stringprep_convert();
    }
    return TRUE;
 }
 
+int ResetInputMethod()
+{
+   int rc=FALSE;
+
+   if (gpIMInfo == NULL) return FALSE;
+
+   sprintf(gszMsgBox, TgLoadString(STID_OK_TO_RESET_GIVEN_IM),
+         gpIMInfo->pszName);
+   if (MsgBox(gszMsgBox, TOOL_NAME, YNC_MB) != MB_ID_YES) {
+      return FALSE;
+   }
+   CleanUpInputMethods();
+   rc = InitInputMethods();
+
+   sprintf(gszMsgBox, TgLoadString(STID_GIVEN_IM_RESETED), gpIMInfo->pszName);
+   Msg(gszMsgBox);
+
+   return rc;
+}
+
+int InputMethodTypeMatched(double_byte)
+   int double_byte;
+{
+   if (gnInputMethodIndex == INVALID || gpIMInfo == NULL) return FALSE;
+
+   if (double_byte) {
+      return (gnSingleOrDoubleByteInputMethod == TGIM_DBIM &&
+            (gpIMInfo->nSingleDouble & TGIM_DBIM) == TGIM_DBIM);
+   } else {
+      return (gnSingleOrDoubleByteInputMethod == TGIM_SBIM &&
+            (gpIMInfo->nSingleDouble & TGIM_SBIM) == TGIM_SBIM);
+   }
+}
+
 int tgIMFocusIn(dpy, win)
    Display *dpy;
    Window win;
