@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/names.c,v 4.48 2001/08/06 21:39:08 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/names.c,v 1.8 2009/08/13 01:14:20 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_NAMES_C_
@@ -57,6 +57,7 @@ char curSymDir[MAXPATHLENGTH+1];
 
 int doubleClickInterval=300;
 int importFromLibrary=FALSE;
+int importingFromExec=FALSE;
 int curDirIsLocal=TRUE;
 
 int ignoreDirectoryFlag=FALSE;
@@ -151,7 +152,7 @@ void ParseSymPath(path_str)
    for (i=0, s=path_str; *s != '\0'; ) {
       s = ReadPath(s, dir_str);
       UtilTrimBlanks(dir_str);
-      if (dir_str != '\0') i++;
+      if (*dir_str != '\0') i++;
    }
    symPath = (char**)malloc(i*sizeof(char*));
    if (symPath == NULL) FailAllocMessage();
@@ -160,7 +161,7 @@ void ParseSymPath(path_str)
    for (i=0, s=path_str; *s != '\0'; ) {
       s = ReadPath(s, dir_str);
       UtilTrimBlanks(dir_str);
-      if (dir_str != '\0') {
+      if (*dir_str != '\0') {
          symPath[i] = (char*)malloc((MAXPATHLENGTH+1)*sizeof(char));
          if (symPath[i] == NULL) FailAllocMessage();
          strcpy(symPath[i], dir_str);
@@ -368,13 +369,13 @@ DspList *DirListing(Path, ExtStr, OtherExtStr)
          if (ext_str != NULL) free(ext_str);
          return NULL;
       }
-#ifdef __EMX__
+#ifdef __CYGWIN__
       if (path[strlen(path)-1] != DIR_SEP) {
          strcat(path, DIR_SEP_STR);
       }
-#else /* ~__EMX__ */
+#else /* ~__CYGWIN__ */
       strcat(path, DIR_SEP_STR);
-#endif /* __EMX__ */
+#endif /* __CYGWIN__ */
    }
 
 #ifdef VMS
@@ -664,7 +665,11 @@ void InitDomain()
          ParseSymPath(c_ptr);
       }
    } else {
-      ParseSymPath(".");
+      char buf[MAXPATHLENGTH];
+
+      strcpy(buf, ".:");
+      UtilStrCpyN(&buf[2], sizeof(buf)-2, TGIF_PATH);
+      ParseSymPath(buf);
    }
    if (need_to_copy_domain_into_ini) {
       CopyDomainInfoToIni();
@@ -686,7 +691,7 @@ void InitNames()
 {
    char *c_ptr=NULL;
 
-   InitNamesInfo();
+   if (mainDisplay != NULL) InitNamesInfo();
 
    ignoreDirectoryFlag = FALSE;
 
@@ -701,6 +706,8 @@ void InitNames()
    strcpy(curLocalDir, bootDir);
    strcpy(curImportDir, bootDir);
 
+   if (mainDisplay == NULL) return;
+
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DoubleClickInterval")) !=
          NULL) {
       doubleClickInterval = atoi(c_ptr);
