@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2003, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.45 2003/02/25 01:17:47 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/navigate.c,v 1.11 2009/01/09 22:25:31 william Exp $
  */
 
 #define _INCLUDE_FROM_NAVIGATE_C_
@@ -23,6 +23,7 @@
 #include "tgifdefs.h"
 #include "cmdids.h"
 
+#include "attr.e"
 #include "auxtext.e"
 #include "button.e"
 #include "choice.e"
@@ -33,6 +34,7 @@
 #include "cutpaste.e"
 #include "dialog.e"
 #include "drawing.e"
+#include "exec.e"
 #include "file.e"
 #include "font.e"
 #include "mainloop.e"
@@ -91,6 +93,7 @@ static int maxURLCache=(-1), curURLCache=0;
 static int lineWidthIndexInSlideShow=4;
 static int makeUnsavableInSlideShow=FALSE;
 static int ignoreSlideShowOffsetsInFile=TRUE;
+static int hideWindowsInSlideShow=TRUE;
 
 static int savedSlideShowXOffset=0;
 static int savedSlideShowYOffset=0;
@@ -360,6 +363,12 @@ void InitNavigate()
          UtilStrICmp(c_ptr, "false") == 0) {
       ignoreSlideShowOffsetsInFile = FALSE;
    }
+   hideWindowsInSlideShow = TRUE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "HideWindowsInSlideShow")) != NULL &&
+         UtilStrICmp(c_ptr, "false") == 0) {
+      hideWindowsInSlideShow = FALSE;
+   }
 }
 
 static
@@ -1268,6 +1277,56 @@ void ToggleHyperSpace(KeepSelected)
    RedrawDummyWindow1();
 }
 
+void NextSlide()
+{
+   struct ObjRec *obj_ptr=FindTopLevelObjWithType("page_defaultplay");
+
+   if (obj_ptr != NULL) {
+      struct AttrRec *exec_attr=FindObjAttrWithName(obj_ptr, "exec=");
+      struct AttrRec *attr_ptr=FindObjAttrWithName(obj_ptr, "finished=");
+
+      if (exec_attr != NULL && attr_ptr != NULL) {
+         char *attr_value=attr_ptr->attr_value.s;
+
+         if (strcmp(attr_value, "0") == 0) {
+            DoExecLoop(obj_ptr, exec_attr);
+            /*
+             * if (!userAbortExec) {
+             *    ReplaceAttrFirstValue(obj_ptr, attr_ptr, "1");
+             * }
+             */
+            return;
+         }
+      }
+   }
+   NextPage();
+}
+
+void PrevSlide()
+{
+   struct ObjRec *obj_ptr=FindTopLevelObjWithType("page_defaultrewind");
+
+   if (obj_ptr != NULL) {
+      struct AttrRec *exec_attr=FindObjAttrWithName(obj_ptr, "exec=");
+      struct AttrRec *attr_ptr=FindObjAttrWithName(obj_ptr, "finished=");
+
+      if (exec_attr != NULL && attr_ptr != NULL) {
+         char *attr_value=attr_ptr->attr_value.s;
+
+         if (strcmp(attr_value, "0") == 0) {
+            DoExecLoop(obj_ptr, exec_attr);
+            /*
+             * if (!userAbortExec) {
+             *    ReplaceAttrFirstValue(obj_ptr, attr_ptr, "1");
+             * }
+             */
+            return;
+         }
+      }
+   }
+   PrevPage();
+}
+
 struct SlideShowInfoRec {
    int saved_in_hyperspace;
    int saved_fill, saved_pen;
@@ -1275,6 +1334,74 @@ struct SlideShowInfoRec {
 };
 static struct SlideShowInfoRec gstSlideShow;
 
+static
+void HideAllWindowsForSlideShow()
+{
+   int i=0;
+
+   if (canvasWindowOnly || !hideWindowsInSlideShow) return;
+
+   XUnmapWindow(mainDisplay, titleWindow);
+   if (menubarWindow != None) XUnmapWindow(mainDisplay, menubarWindow);
+   if (!noChoiceWindow) XUnmapWindow(mainDisplay, msgWindow);
+   if (!noChoiceWindow) XUnmapWindow(mainDisplay, choiceWindow);
+
+   XUnmapWindow(mainDisplay, hRuleWindow);
+   XUnmapWindow(mainDisplay, vRuleWindow);
+
+   XUnmapWindow(mainDisplay, colorWindow);
+   XUnmapWindow(mainDisplay, colorDummyWindow);
+   XUnmapWindow(mainDisplay, vSBarWindow);
+   XUnmapWindow(mainDisplay, pageWindow);
+   XUnmapWindow(mainDisplay, pageDummyWindow);
+   XUnmapWindow(mainDisplay, hSBarWindow);
+   XUnmapWindow(mainDisplay, dummyWindow1);
+   XUnmapWindow(mainDisplay, dummyWindow2);
+   if (!noChatWindow) XUnmapWindow(mainDisplay, chatWindow);
+   if (!noStatusWindow) {
+      XUnmapWindow(mainDisplay, userRedrawWindow);
+      XUnmapWindow(mainDisplay, statusWindow);
+      for (i=0; i < MAX_STATUS_BTNS; i++) {
+         XUnmapWindow(mainDisplay, statusSubWindow[i]);
+      }
+   }
+   if (!noModeWindow) XUnmapWindow(mainDisplay, modeWindow);
+}
+
+static
+void ShowAllWindowsForSlideShow()
+{
+   int i=0;
+
+   if (canvasWindowOnly || !hideWindowsInSlideShow) return;
+
+   XMapWindow(mainDisplay, titleWindow);
+   if (menubarWindow != None) XMapWindow(mainDisplay, menubarWindow);
+   if (!noChoiceWindow) XMapWindow(mainDisplay, msgWindow);
+   if (!noChoiceWindow) XMapWindow(mainDisplay, choiceWindow);
+
+   XMapWindow(mainDisplay, hRuleWindow);
+   XMapWindow(mainDisplay, vRuleWindow);
+
+   XMapWindow(mainDisplay, colorWindow);
+   XMapWindow(mainDisplay, colorDummyWindow);
+   XMapWindow(mainDisplay, vSBarWindow);
+   XMapWindow(mainDisplay, pageWindow);
+   XMapWindow(mainDisplay, pageDummyWindow);
+   XMapWindow(mainDisplay, hSBarWindow);
+   XMapWindow(mainDisplay, dummyWindow1);
+   XMapWindow(mainDisplay, dummyWindow2);
+   if (!noChatWindow) XMapWindow(mainDisplay, chatWindow);
+   if (!noStatusWindow) {
+      XMapWindow(mainDisplay, userRedrawWindow);
+      XMapWindow(mainDisplay, statusWindow);
+      for (i=0; i < MAX_STATUS_BTNS; i++) {
+         XMapWindow(mainDisplay, statusSubWindow[i]);
+      }
+   }
+   if (!noModeWindow) XMapWindow(mainDisplay, modeWindow);
+}
+
 static
 void StartSlideShow()
 {
@@ -1311,6 +1438,7 @@ void StartSlideShow()
    }
    FixPageNumbersForSlideShow();
    ClearObjCachesInAllPages();
+   HideAllWindowsForSlideShow();
    HidePopupMenusForSlideShow();
 
    RecordWBSlideShow(TRUE);
@@ -1327,6 +1455,7 @@ void EndSlideShow()
 
    RecordWBSlideShow(FALSE);
 
+   ShowAllWindowsForSlideShow();
    ShowPopupMenusForSlideShow();
    ClearObjCachesInAllPages();
    if (gstSlideShow.saved_in_hyperspace != inHyperSpace) {
