@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2006, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2009, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/rcbox.c,v 1.3 2006/02/10 23:21:20 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/rcbox.c,v 1.9 2009/09/28 17:43:48 william Exp $
  */
 
 #define _INCLUDE_FROM_RCBOX_C_
@@ -214,7 +214,7 @@ void DumpRCBoxPath(FP, ObjPtr, ltx, lty, rbx, rby, r, width, pen, dash,
       if (colorDump || !useGray) {
          if (preDumpSetup) PSUseColorPattern();
          fprintf(FP, "      %s\n", gPsCmd[PS_FLATTENPATH]);
-         DumpPatFill(FP, pen, 8, ObjPtr->bbox, 6, TRUE);
+         DumpPatFill(FP, pen, ObjPtr->bbox, 6, TRUE);
       } else {
          fprintf(FP, "      %s\n", gPsCmd[PS_STROKE]);
       }
@@ -297,7 +297,7 @@ void DumpRCBoxObj(FP, ObjPtr)
          if (preDumpSetup) PSUseColorPattern();
          fprintf(FP, "   %s %s %s\n", gPsCmd[PS_CLOSEPATH],
                gPsCmd[PS_EOCLIP], gPsCmd[PS_NEWPATH]);
-         DumpPatFill(FP, fill, 8, ObjPtr->bbox, 3, TRUE);
+         DumpPatFill(FP, fill, ObjPtr->bbox, 3, TRUE);
       } else {
          GrayCheck(fill);
          fprintf(FP, "   %s %s\n", GrayStr(fill), gPsCmd[PS_SETGRAY]);
@@ -397,8 +397,8 @@ void MakeCachedRCBox(ObjPtr)
       rcbox_ptr->rotated_vlist[i].y = (short)OFFSET_Y(pv[i].y);
    }
    if (num_vs == 13) {
-      sv = MakeMultiSplinePolygonVertex(&sn, smooth, drawOrigX, drawOrigY,
-            num_vs, pv);
+      sv = MakeMultiSplinePolygonVertex(LT_SPLINE, &sn, smooth, drawOrigX,
+            drawOrigY, num_vs, pv);
       if (sv == NULL) FailAllocMessage();
       free(rcbox_ptr->rotated_vlist);
       rcbox_ptr->rotated_n = sn;
@@ -548,6 +548,10 @@ void CreateRCBoxObj(X1, Y1, X2, Y2, CreateAbsolute)
    obj_ptr->bbox.rby += w;
    obj_ptr->type = OBJ_RCBOX;
    obj_ptr->color = colorIndex;
+   if (mainDisplay != NULL) {
+      UtilStrCpyN(obj_ptr->color_str, sizeof(obj_ptr->color_str),
+            colorMenuItems[colorIndex]);
+   }
    obj_ptr->id = objId++;
    obj_ptr->dirty = FALSE;
    obj_ptr->rotation = 0;
@@ -881,6 +885,7 @@ void ReadRCBoxObj(FP, Inbuf, ObjPtr)
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
    (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
+   UtilStrCpyN((*ObjPtr)->color_str, sizeof((*ObjPtr)->color_str), color_str);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -905,7 +910,7 @@ void ReadRCBoxObj(FP, Inbuf, ObjPtr)
       char inbuf[MAXSTRING+1];
       struct XfrmMtrxRec *ctm;
 
-      fgets(inbuf, MAXSTRING, FP);
+      (void)fgets(inbuf, MAXSTRING, FP);
       scanLineNum++;
       InitScan(inbuf, "\t\n, ");
 
