@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ruler.c,v 1.5 2009/01/09 22:25:31 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/ruler.c,v 1.19 2009/10/21 23:20:08 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_RULER_C_
@@ -25,15 +25,20 @@
 #include "choice.e"
 #include "cursor.e"
 #include "dialog.e"
+#include "file.e"
 #include "font.e"
 #include "grid.e"
+#include "mainloop.e"
 #include "mainmenu.e"
+#include "menu.e"
 #include "msg.e"
 #include "raster.e"
+#include "rect.e"
 #include "ruler.e"
 #include "select.e"
 #include "setup.e"
 #include "strtbl.e"
+#include "text.e"
 #include "util.e"
 #include "xpixmap.e"
 
@@ -316,6 +321,117 @@ void SquarePixelToMeasurementUnit(Buf, NumSquarePixels)
    }
 }
 
+void PixelToCurrentUnit(Buf, NumPixels)
+   char *Buf;
+   int NumPixels;
+{
+   int unit_str_was_empty=(*unitStr == '\0');
+   char num_unit_str[80], base_unit_str[80];
+   float pixel_per_unit=gfPixelsPerUnit, num_units=gfNumUnits;
+
+   if (unit_str_was_empty) {
+      UtilStrCpyN(num_unit_str, sizeof(num_unit_str), numUnitStr);
+      UtilStrCpyN(base_unit_str, sizeof(base_unit_str), baseUnitStr);
+      switch (gridSystem) {
+      case ENGLISH_GRID:
+         gfPixelsPerUnit = 128;
+         gfNumUnits = 1;
+         UtilStrCpyN(unitStr, sizeof(unitStr), "inch");
+         break;
+      case METRIC_GRID:
+         gfPixelsPerUnit = 50;
+         gfNumUnits = 2.5;
+         UtilStrCpyN(unitStr, sizeof(unitStr), "cm");
+         break;
+      }
+      PixelToMeasurementUnit(Buf, NumPixels);
+   } else {
+      PixelToMeasurementUnit(Buf, NumPixels);
+   }
+   if (unit_str_was_empty) {
+      gfPixelsPerUnit = pixel_per_unit;
+      gfNumUnits = num_units;
+      *unitStr = '\0';
+      UtilStrCpyN(numUnitStr, sizeof(numUnitStr), num_unit_str);
+      UtilStrCpyN(baseUnitStr, sizeof(baseUnitStr), base_unit_str);
+   }
+}
+
+static
+int RightMarginMarkVisible()
+{
+   return ((rightMarginEnabled == TRUE) && (drawOrigX+drawWinW >
+         paperWidth-(rightMargin>>TIK_PER_PIXEL_SHIFTS)));
+}
+
+static
+void GetRightMarginMarkVs(vs)
+   XPoint vs[6];
+{
+   int x=OFFSET_X(paperWidth-(rightMargin>>TIK_PER_PIXEL_SHIFTS))-1;
+   int delta=(rulerW>>2);
+   int y_end=rulerW-(threeDLook ? ((windowPadding>>1)+2) : 1);
+   int y_start=(threeDLook ? ((windowPadding>>1)-2) : (-1))+(delta<<1);
+
+   vs[0].x = x;        vs[0].y = y_start;
+   vs[1].x = x-delta;  vs[1].y = y_start+delta;
+   vs[2].x = x-delta;  vs[2].y = y_end;
+   vs[3].x = x+delta;  vs[3].y = y_end;
+   vs[4].x = x+delta;  vs[4].y = y_start+delta;
+   vs[5].x = x;        vs[5].y = y_start;
+}
+
+static
+int PtInRightMarginMark(button_x, button_y)
+   int button_x, button_y;
+{
+   XPoint vs[6];
+
+   GetRightMarginMarkVs(vs);
+
+   return PointInPolygon(button_x, button_y, 6, vs);
+}
+
+static
+void RedrawRightMargin(dpy, win)
+   Display *dpy;
+   Window win;
+{
+   if (rightMarginEnabled != TRUE) return;
+
+   if (drawOrigX+drawWinW > paperWidth-(rightMargin>>TIK_PER_PIXEL_SHIFTS)) {
+      int delta=(rulerW>>2);
+      int x=OFFSET_X(paperWidth-(rightMargin>>TIK_PER_PIXEL_SHIFTS))-1;
+      int y_end=rulerW-(threeDLook ? ((windowPadding>>1)+2) : 1);
+      int y_start=(threeDLook ? ((windowPadding>>1)-2) : (-1))+(delta<<1);
+      int bg_pixel=(threeDLook ? myLtGryPixel : myBgPixel);
+      GC gc=defaultGC;
+      XPoint vs[6];
+
+      GetRightMarginMarkVs(vs);
+      if (threeDLook) {
+         XSetForeground(mainDisplay, textMenuGC, bg_pixel);
+         XFillPolygon(dpy, win, textMenuGC, vs, 6, Convex, CoordModeOrigin);
+         XDrawLines(dpy, win, gc, vs, 6, CoordModeOrigin);
+      } else {
+         XDrawLines(dpy, win, gc, vs, 6, CoordModeOrigin);
+      }
+      rightMarginActive = ShouldRightMarginBeActive();
+      if (!rightMarginActive) {
+         XDrawLine(dpy, win, gc, x-delta, y_start+1, x+delta-1, y_end);
+         XDrawLine(dpy, win, gc, x-delta+1, y_start+1, x+delta, y_end);
+         /*
+          * The code doesn't look right, but it look symmetrical on the screen!
+          *
+          * XDrawLine(dpy, win, gc, x+delta, y_start+1, x-delta+1, y_end);
+          * XDrawLine(dpy, win, gc, x+delta-1, y_start+1, x-delta, y_end);
+          */
+         XDrawLine(dpy, win, gc, x+delta+1, y_start, x-delta+1, y_end);
+         XDrawLine(dpy, win, gc, x+delta, y_start, x-delta, y_end);
+      }
+   }
+}
+
 void RedrawHRuler(dpy, win)
    Display *dpy;
    Window win;
@@ -382,6 +498,7 @@ void RedrawHRuler(dpy, win)
       }
       break;
    }
+   RedrawRightMargin(dpy, win);
 }
 
 static
@@ -633,13 +750,108 @@ void ToggleShowCrossHair()
    }
 }
 
+static
+void ContinueMoveRightMarginMark(dpy, OrigX, OrigY)
+   Display *dpy;
+   int OrigX, OrigY; /* these are mouse coordinates */
+{
+   int moving=TRUE, dx=0, grid_x=0, grid_y=0, h=ZOOMED_SIZE(drawWinH);
+   int prev_x=0, orig_grid_x=0, orig_grid_y=0, orig_right_margin=0;
+   char buf[80], buf2[80];
+   double dval=(double)0;
+   XPoint orig_vs[6], vs[6];
+   XEvent ev;
+
+   XFlush(dpy);
+   XSync(dpy, False);
+
+   if (XCheckMaskEvent(dpy, ExposureMask, &ev) ||
+         XCheckMaskEvent(dpy, VisibilityChangeMask, &ev)) {
+      ExposeEventHandler(&ev, TRUE);
+   }
+   GridXY(OrigX, OrigY, &grid_x, &grid_y);
+   orig_grid_x = grid_x;
+   orig_grid_y = grid_y;
+   orig_right_margin = OFFSET_X(paperWidth-(rightMargin>>TIK_PER_PIXEL_SHIFTS));
+   prev_x = orig_right_margin+0;
+   GetRightMarginMarkVs(orig_vs);
+   memcpy(vs, orig_vs, 6*sizeof(XPoint));
+   dval = ((double)rightMargin) * ((double)printMag) / ((double)TIK_PER_PIXEL);
+   dval /= ((double)100);
+   PixelToMeasurementUnit(buf, round(dval));
+   sprintf(buf2, "margin=%s", buf);
+   StartShowMeasureCursor(OrigX, 4, buf2, FALSE);
+
+   /* draw */
+   XDrawLine(dpy, drawWindow, revDefaultGC, prev_x, 0, prev_x, h);
+   XDrawLines(dpy, hRuleWindow, revDefaultGC, vs, 6, CoordModeOrigin);
+   if (!debugNoPointerGrab) {
+      XGrabPointer(dpy, drawWindow, FALSE,
+            PointerMotionMask | ButtonReleaseMask,
+            GrabModeAsync, GrabModeAsync, None, moveCursor, CurrentTime);
+   }
+   while (moving) {
+      XEvent input;
+
+      XNextEvent(dpy, &input);
+
+      if (input.type == Expose || input.type == VisibilityNotify) {
+         ExposeEventHandler(&input, TRUE);
+      } else if (input.type == ButtonRelease) {
+         int abs_dx=ABS_SIZE(dx);
+
+         XUngrabPointer(dpy, CurrentTime);
+         XSync(dpy, False);
+
+         moving = FALSE;
+
+         /* erase */
+         XDrawLine(dpy, drawWindow, revDefaultGC, prev_x, 0, prev_x, h);
+         XDrawLines(dpy, hRuleWindow, revDefaultGC, vs, 6, CoordModeOrigin);
+         EndShowMeasureCursor(OrigX+dx, 4, buf2, FALSE);
+
+         rightMargin -= (abs_dx<<TIK_PER_PIXEL_SHIFTS);
+         RedrawHRulerWindow();
+      } else if (input.type == MotionNotify) {
+         int i=0, mouse_x=0, mouse_y=0;
+
+         /* erase */
+         XDrawLine(dpy, drawWindow, revDefaultGC, prev_x, 0, prev_x, h);
+         XDrawLines(dpy, hRuleWindow, revDefaultGC, vs, 6, CoordModeOrigin);
+         ShowMeasureCursor(OrigX+dx, 4, buf2, FALSE);
+
+         mouse_x = input.xmotion.x;
+         mouse_y = input.xmotion.y;
+         GridXY(mouse_x, mouse_y, &grid_x, &grid_y);
+
+         dx = grid_x - orig_grid_x;
+         prev_x = orig_right_margin+0+dx;
+         for (i=0; i < 6; i++) {
+            vs[i].x = orig_vs[i].x+dx;
+         }
+         /* draw */
+         XDrawLine(dpy, drawWindow, revDefaultGC, prev_x, 0, prev_x, h);
+         XDrawLines(dpy, hRuleWindow, revDefaultGC, vs, 6, CoordModeOrigin);
+
+         dval = (((double)(rightMargin-(ABS_SIZE(dx)<<TIK_PER_PIXEL_SHIFTS))) *
+               ((double)printMag) / ((double)TIK_PER_PIXEL)) / ((double)100);
+         PixelToMeasurementUnit(buf, round(dval));
+         sprintf(buf2, "margin=%s", buf);
+         ShowMeasureCursor(OrigX+dx, 4, buf2, FALSE);
+      }
+   }
+}
+
 void RulersEventHandler(input)
    XEvent *input;
 {
    if (input->type == ButtonPress) {
       XButtonEvent *button_ev=(&input->xbutton);
+      int mouse_x=button_ev->x, mouse_y=button_ev->y;
 
-      if (button_ev->button == Button1) {
+      if (RightMarginMarkVisible() && PtInRightMarginMark(mouse_x, mouse_y)) {
+         ContinueMoveRightMarginMark(mainDisplay, mouse_x, mouse_y);
+      } else if (button_ev->button == Button1) {
          IncGrid();
       } else if (button_ev->button == Button2) {
          GridMenu(button_ev->x_root, button_ev->y_root, FALSE);
