@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/shortcut.c,v 1.33 2009/09/28 22:39:48 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/shortcut.c,v 1.36 2009/10/22 00:00:17 cvsps Exp $
  */
 
 #define _INCLUDE_FROM_SHORTCUT_C_
@@ -353,6 +353,7 @@ static struct ShortCutRec shortCutXlateTbl[] = {
                CMDID_SETALTEDITTEXTHIGHLIGHTCOLOR },
    { '\0', 0, "ToggleAltEditTextBgColor()", 0, CMDID_TOGGLEALTEDITTEXTBGCOLOR },
    { '\0', 0, "ChangeAllSelFontUnderline()",0,CMDID_CHANGEALLSELFONTUNDERLINE },
+   { '\0', 0, "ChangeAllSelFontOverline()",0,CMDID_CHANGEALLSELFONTOVERLINE },
    { '\0', 0, "EditPageFileNames()", 0, CMDID_EDITPAGEFILENAMES },
    { '\0', 0, "ExportHalfToneBitmap()", 0, CMDID_EXPORTHALFTONEBITMAP },
    { '\0', 0, "ExportThresholdBitmap()", 0, CMDID_EXPORTTHRESHOLDBITMAP },
@@ -456,6 +457,9 @@ static struct ShortCutRec shortCutXlateTbl[] = {
    { '\0', 0, "ChooseRotatePivot()", 0, CMDID_CHOOSEROTATEPIVOT },
    { '\0', 0, "ChooseColor()", 0, CMDID_COLORWHEEL },
    { '\0', 0, "ExtendSegment()", 0, CMDID_EXTENDSEGMENT },
+   { '\0', 0, "InsertSymbol()", 0, CMDID_INSERTSYMBOL },
+   { '\0', 0, "ToggleRightMargin()", 0, CMDID_TOGGLERIGHTMARGIN },
+   { '\0', 0, "SpecifyRightMargin()", 0, CMDID_SPECIFYRIGHTMARGIN },
    { '\0', 0, "ToggleTighterStructuredSplines()", 0, CMDID_TOGGLETIGHTERSPLINES },
    { '\0', 0, "", 0, 0 }
 };
